@@ -118,7 +118,6 @@ pub fn start_client(
         palette,
     };
 
-    #[cfg(not(any(feature = "test", test)))]
     let first_msg = match info {
         ClientInfo::Attach(name, force, config_options) => {
             SESSION_NAME.set(name).unwrap();
@@ -140,16 +139,6 @@ pub fn start_client(
             )
         }
     };
-    #[cfg(any(feature = "test", test))]
-    let first_msg = {
-        let _ = SESSION_NAME.set("".into());
-        ClientToServerMsg::NewClient(
-            client_attributes,
-            Box::new(opts),
-            Box::new(config_options.clone()),
-            layout,
-        )
-    };
 
     os_input.connect_to_server(&*ZELLIJ_IPC_PIPE);
     os_input.send_to_server(first_msg);
@@ -167,7 +156,6 @@ pub fn start_client(
     > = channels::bounded(50);
     let send_client_instructions = SenderWithContext::new(send_client_instructions);
 
-    #[cfg(not(any(feature = "test", test)))]
     std::panic::set_hook({
         use zellij_utils::errors::handle_panic;
         let send_client_instructions = send_client_instructions.clone();
@@ -176,6 +164,8 @@ pub fn start_client(
         })
     });
 
+    let on_force_close = config_options.on_force_close.unwrap_or_default();
+
     let _stdin_thread = thread::Builder::new()
         .name("stdin_handler".to_string())
         .spawn({
@@ -212,7 +202,7 @@ pub fn start_client(
                     Box::new({
                         let os_api = os_input.clone();
                         move || {
-                            os_api.send_to_server(ClientToServerMsg::Action(Action::Detach));
+                            os_api.send_to_server(ClientToServerMsg::Action(on_force_close.into()));
                         }
                     }),
                 );
