@@ -8,7 +8,11 @@
 //  place.
 //  If plugins should be able to depend on the layout system
 //  then [`zellij-utils`] could be a proper place.
-use crate::{input::config::ConfigError, pane_size::PositionAndSize, setup};
+use crate::{
+    input::{command::RunCommand, config::ConfigError},
+    pane_size::PositionAndSize,
+    setup,
+};
 use crate::{serde, serde_yaml};
 
 use serde::{Deserialize, Serialize};
@@ -29,6 +33,15 @@ pub enum SplitSize {
     Fixed(u16),  // An absolute number of columns or rows
 }
 
+#[derive(Debug, Serialize, Deserialize, Clone)]
+#[serde(crate = "self::serde")]
+pub enum Run {
+    #[serde(rename = "plugin")]
+    Plugin(Option<PathBuf>),
+    #[serde(rename = "command")]
+    Command(RunCommand),
+}
+
 #[derive(Debug, Serialize, Deserialize, Clone)]
 #[serde(crate = "self::serde")]
 pub struct Layout {
@@ -36,7 +49,7 @@ pub struct Layout {
     #[serde(default)]
     pub parts: Vec<Layout>,
     pub split_size: Option<SplitSize>,
-    pub plugin: Option<PathBuf>,
+    pub run: Option<Run>,
 }
 
 type LayoutResult = Result<Layout, ConfigError>;
@@ -127,13 +140,28 @@ impl Layout {
         let mut total_panes = 0;
         total_panes += self.parts.len();
         for part in self.parts.iter() {
-            if part.plugin.is_none() {
-                total_panes += part.total_terminal_panes();
+            match part.run {
+                Some(Run::Command(_)) | None => {
+                    total_panes += part.total_terminal_panes();
+                }
+                Some(Run::Plugin(_)) => {}
             }
         }
         total_panes
     }
 
+    pub fn extract_run_instructions(&self) -> Vec<Option<Run>> {
+        let mut run_instructions = vec![];
+        if self.parts.is_empty() {
+            run_instructions.push(self.run.clone());
+        }
+        for part in self.parts.iter() {
+            let mut current_runnables = part.extract_run_instructions();
+            run_instructions.append(&mut current_runnables);
+        }
+        run_instructions
+    }
+
     pub fn position_panes_in_space(
         &self,
         space: &PositionAndSize,
